@@ -36,6 +36,7 @@
 #include "x11fwd.h"
 #include "agentfwd.h"
 #include "localtcpfwd.h"
+#include "remotetcpfwd.h"
 
 static void send_msg_channel_open_failure(unsigned int remotechan, int reason,
 		const unsigned char *text, const unsigned char *lang);
@@ -68,7 +69,6 @@ void chaninitialise() {
 	/* may as well create space for a single channel */
 	ses.channels = (struct Channel**)m_malloc(sizeof(struct Channel*));
 	ses.chansize = 1;
-	ses.chancount = 0;
 	ses.channels[0] = NULL;
 
 	chansessinitialise();
@@ -111,7 +111,7 @@ struct Channel* newchannel(unsigned int remotechan, unsigned char type,
 
 	/* otherwise extend the list */
 	if (i == ses.chansize) {
-		if (ses.chansize > MAX_CHANNELS) {
+		if (ses.chansize >= MAX_CHANNELS) {
 			TRACE(("leave newchannel: max chans reached"));
 			return NULL;
 		}
@@ -150,7 +150,6 @@ struct Channel* newchannel(unsigned int remotechan, unsigned char type,
 	newchan->recvmaxpacket = RECV_MAXPACKET;
 
 	ses.channels[i] = newchan;
-	ses.chancount++;
 
 	TRACE(("leave newchannel"));
 
@@ -224,6 +223,9 @@ void channelio(fd_set *readfd, fd_set *writefd) {
 			if (chansess->agentfd >= 0 && FD_ISSET(chansess->agentfd,readfd)) {
 				agentaccept(chansess);
 			}
+#endif
+#ifndef DISABLE_REMOTETCPFWD
+			handleremotetcp(readfd);
 #endif
 		}
 
@@ -259,7 +261,21 @@ static void checkclose(struct Channel *channel) {
 		}
 	}
 
-	if (channel->sentclosed && channel->recvclosed) {
+	/* When either party wishes to terminate the channel, it sends
+	 * SSH_MSG_CHANNEL_CLOSE.  Upon receiving this message, a party MUST
+	 * send back a SSH_MSG_CHANNEL_CLOSE unless it has already sent this
+	 * message for the channel.  The channel is considered closed for a
+	 * party when it has both sent and received SSH_MSG_CHANNEL_CLOSE, and
+	 * the party may then reuse the channel number.  A party MAY send
+	 * SSH_MSG_CHANNEL_CLOSE without having sent or received
+	 * SSH_MSG_CHANNEL_EOF. 
+	 * (from draft-ietf-secsh-connect)
+	 */
+	if (channel->recvclosed) {
+		if (! channel->sentclosed) {
+			TRACE(("Sending MSG_CHANNEL_CLOSE in response to same."));
+			send_msg_channel_close(channel);
+		}
 		removechannel(channel);
 	}
 }
@@ -357,8 +373,7 @@ static void writechannel(struct Channel* channel) {
 
 	len = write(channel->infd, buf_getptr(buf, maxlen), maxlen);
 	if (len <= 0) {
-		if (errno != EINTR) {
-
+		if (len < 0 && errno != EINTR) {
 			/* no more to write */
 			closeinfd(channel);
 		}
@@ -440,6 +455,9 @@ void setchannelfds(fd_set *readfd, fd_set *writefd) {
 			if (chansess->agentfd >= 0) {
 				FD_SET(chansess->agentfd, readfd);
 			}
+#endif
+#ifndef DISABLE_REMOTETCPFWD
+			setremotetcpfds(readfd);
 #endif
 		}
 	} /* foreach channel */
@@ -527,7 +545,6 @@ static void removechannel(struct Channel * channel) {
 static void deletechannel(struct Channel *channel) {
 
 	ses.channels[channel->index] = NULL;
-	ses.chancount--;
 	m_free(channel);
 	
 }
@@ -610,8 +627,8 @@ static void send_msg_channel_data(struct Channel *channel, int isextended,
 	buf = buf_new(maxlen);
 	len = read(fd, buf_getwriteptr(buf, maxlen), maxlen);
 	if (len <= 0) {
-		/* on error etc, send eof */
-		if (errno != EINTR) {
+		/* on error/eof, send eof */
+		if (len == 0 || errno != EINTR) {
 			closeoutfd(channel, fd);
 			TRACE(("leave send_msg_channel_data: read err"));
 		}
@@ -783,7 +800,10 @@ void recv_msg_channel_open() {
 		newchansess(channel);
 #ifndef DISABLE_LOCALTCPFWD
 	} else if (typeval == CHANNEL_ID_TCPDIRECT) {
-		if (newtcpdirect(channel) == DROPBEAR_FAILURE) {
+		if (ses.opts->nolocaltcp) {
+			errtype = SSH_OPEN_ADMINISTRATIVELY_PROHIBITED;
+		} else if (newtcpdirect(channel) == DROPBEAR_FAILURE) {
+			errtype = SSH_OPEN_CONNECT_FAILED;
 			deletechannel(channel);
 			goto failure;
 		}
@@ -873,17 +893,21 @@ static void send_msg_channel_open_confirmation(struct Channel* channel,
  * options, with the calling function calling encrypt_packet() after
  * completion. It is mandatory for the caller to encrypt_packet() if
  * DROPBEAR_SUCCESS is returned */
-int send_msg_channel_open_init(int fd, const char * typestring) {
+int send_msg_channel_open_init(int fd, unsigned char type,
+		const char * typestring) {
 
 	struct Channel* chan;
 
-	chan = newchannel(0, CHANNEL_ID_AGENT, 0, 0);
+	TRACE(("enter send_msg_channel_open_init()"));
+	chan = newchannel(0, type, 0, 0);
 	if (!chan) {
+		TRACE(("leave send_msg_channel_open_init() - FAILED in newchannel()"));
 		return DROPBEAR_FAILURE;
 	}
 
 	/* set fd non-blocking */
 	if (fcntl(fd, F_SETFL, O_NONBLOCK) < 0) {
+		TRACE(("leave send_msg_channel_open_init() - FAILED in fcntl()"));
 		return DROPBEAR_FAILURE;
 	}
 
@@ -899,6 +923,7 @@ int send_msg_channel_open_init(int fd, const char * typestring) {
 	buf_putint(ses.writepayload, RECV_MAXWINDOW);
 	buf_putint(ses.writepayload, RECV_MAXPACKET);
 
+	TRACE(("leave send_msg_channel_open_init()"));
 	return DROPBEAR_SUCCESS;
 }
 
