@@ -40,7 +40,7 @@
 /* Handles sessions (either shells or programs) requested by the client */
 
 static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
-		char iscmd);
+		int iscmd, int issubsys);
 static int sessionpty(struct ChanSess * chansess);
 static int sessionsignal(struct ChanSess *chansess);
 static int noptycommand(struct Channel *channel, struct ChanSess *chansess);
@@ -109,10 +109,8 @@ static void sesssigchild_handler(int val) {
 
 		/* find the corresponding chansess */
 		for (i = 0; i < ses.childpidsize; i++) {
-			assert(pid > 1);
 			if (ses.childpids[i].pid == pid) {
 
-				assert(ses.childpids[i].chansess != NULL);
 				chansess = ses.childpids[i].chansess;
 				chansess->exited = 1;
 				if (WIFEXITED(status)) {
@@ -169,13 +167,11 @@ void send_msg_chansess_exitsignal(struct Channel * channel,
 
 	/* we check that we can match a signal name, otherwise
 	 * don't send anything */
-	i = 0;
-	while (signames[i].name != 0) {
+	for (i = 0; signames[i].name != NULL; i++) {
 		if (signames[i].signal == chansess->exitsignal) {
 			signame = signames[i].name;
 			break;
 		}
-		i++;
 	}
 
 	if (signame == NULL) {
@@ -274,8 +270,8 @@ void closechansess(struct Channel *channel) {
 	for (i = 0; i < ses.childpidsize; i++) {
 		if (ses.childpids[i].chansess == chansess) {
 			assert(ses.childpids[i].pid > 0);
-			TRACE(("closing pid %d\n", ses.childpids[i].pid));
-			TRACE(("exited = %d\n", chansess->exited));
+			TRACE(("closing pid %d", ses.childpids[i].pid));
+			TRACE(("exited = %d", chansess->exited));
 			ses.childpids[i].pid = -1;
 			ses.childpids[i].chansess = NULL;
 		}
@@ -304,24 +300,24 @@ void chansessionrequest(struct Channel *channel) {
 	wantreply = buf_getbyte(ses.payload);
 
 	if (typelen > MAX_NAME_LEN) {
-		send_msg_channel_failure(channel);
-		m_free(type);
 		TRACE(("leave chansessionrequest: type too long")); /* XXX send error?*/
-		return;
+		goto out;
 	}
 
 	chansess = (struct ChanSess*)channel->typedata;
 	assert(chansess != NULL);
-	TRACE(("type is %s\n", type));
+	TRACE(("type is %s", type));
 
 	if (strcmp(type, "window-change") == 0) {
 		ret = sessionwinchange(chansess);
 	} else if (strcmp(type, "shell") == 0) {
-		ret = sessioncommand(channel, chansess, 0);
+		ret = sessioncommand(channel, chansess, 0, 0);
 	} else if (strcmp(type, "pty-req") == 0) {
 		ret = sessionpty(chansess);
 	} else if (strcmp(type, "exec") == 0) {
-		ret = sessioncommand(channel, chansess, 1);
+		ret = sessioncommand(channel, chansess, 1, 0);
+	} else if (strcmp(type, "subsystem") == 0) {
+		ret = sessioncommand(channel, chansess, 1, 1);
 #ifndef DISABLE_X11FWD
 	} else if (strcmp(type, "x11-req") == 0) {
 		ret = x11req(chansess);
@@ -336,6 +332,8 @@ void chansessionrequest(struct Channel *channel) {
 		/* etc, todo "env", "subsystem" */
 	}
 
+out:
+
 	if (wantreply) {
 		if (ret == 0) {
 			send_msg_channel_success(channel);
@@ -450,16 +448,31 @@ static int sessionpty(struct ChanSess * chansess) {
 		unsigned char opcode;
 		unsigned int value;
 		const struct TermCode * termcode;
+		unsigned int len;
+
+		len = buf_getint(ses.payload);
+		if (len != ses.payload->len - ses.payload->pos) {
+			dropbear_exit("bad term mode string");
+		}
+
+		if (len == 0) {
+			TRACE(("empty terminal modes string"));
+			return DROPBEAR_SUCCESS;
+		}
 
 		while (((opcode = buf_getbyte(ses.payload)) != 0x00) &&
 				opcode <= 159) {
+
+			/* must be before checking type, so that value is consumed even if
+			 * we don't use it */
+			value = buf_getint(ses.payload);
+
 			/* handle types of code */
 			if (opcode > MAX_TERMCODE) {
 				continue;
 			}
 			termcode = &termcodes[(unsigned int)opcode];
 			
-			value = buf_getint(ses.payload);
 
 			switch (termcode->type) {
 
@@ -518,7 +531,7 @@ static int sessionpty(struct ChanSess * chansess) {
  * noptycommand or ptycommand as appropriate.
  * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
-		char iscmd) {
+		int iscmd, int issubsys) {
 
 	unsigned int cmdlen;
 
@@ -537,6 +550,17 @@ static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
 			/* TODO - send error - too long ? */
 			return DROPBEAR_FAILURE;
 		}
+		if (issubsys) {
+#ifdef SFTPSERVER_PATH
+			if ((cmdlen == 4) && strncmp(chansess->cmd, "sftp", 4) == 0) {
+				m_free(chansess->cmd);
+				chansess->cmd = strdup(SFTPSERVER_PATH);
+			} else 
+#endif
+			{
+				return DROPBEAR_FAILURE;
+			}
+		}
 	}
 
 	if (chansess->term == NULL) {
@@ -633,11 +657,21 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 	pid_t pid;
 	struct logininfo *li;
+#ifdef DO_MOTD
+	buffer * motdbuf = NULL;
+	int len;
+	struct stat sb;
+	char *hushpath = NULL;
+#endif
 
 	TRACE(("enter ptycommand"));
 
-	/* we already have a pty allocated */
-	assert(chansess->master != -1 && chansess->tty != NULL);
+	/* we need to have a pty allocated */
+	if (chansess->master == -1 || chansess->tty == NULL) {
+		dropbear_log(LOG_WARNING, "no pty was allocated, couldn't execute");
+		return DROPBEAR_FAILURE;
+	}
+	
 	pid = fork();
 	if (pid < 0)
 		return DROPBEAR_FAILURE;
@@ -668,6 +702,34 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 		m_free(chansess->tty);
 
+#ifdef DO_MOTD
+		if (ses.opts->domotd) {
+			/* don't show the motd if ~/.hushlogin exists */
+
+			/* 11 == strlen("/hushlogin\0") */
+			len = strlen(ses.authstate.pw->pw_dir) + 11; 
+
+			hushpath = m_malloc(len);
+			snprintf(hushpath, len, "%s/hushlogin", ses.authstate.pw->pw_dir);
+
+			if (stat(hushpath, &sb) < 0) {
+				/* more than a screenful is stupid IMHO */
+				motdbuf = buf_new(80 * 25);
+				if (buf_readfile(motdbuf, MOTD_FILENAME) == DROPBEAR_SUCCESS) {
+					buf_setpos(motdbuf, 0);
+					while (motdbuf->pos != motdbuf->len) {
+						len = motdbuf->len - motdbuf->pos;
+						len = write(STDOUT_FILENO, 
+								buf_getptr(motdbuf, len), len);
+						buf_incrpos(motdbuf, len);
+					}
+				}
+				buf_free(motdbuf);
+			}
+			m_free(hushpath);
+		}
+#endif /* DO_MOTD */
+
 		execchild(chansess);
 		/* not reached */
 
@@ -709,6 +771,7 @@ static void addchildpid(struct ChanSess *chansess, pid_t pid) {
 	if (i == ses.childpidsize) {
 		ses.childpids = (struct ChildPid*)m_realloc(ses.childpids,
 				sizeof(struct ChildPid) * ses.childpidsize+1);
+		ses.childpidsize++;
 	}
 	
 	ses.childpids[i].pid = pid;
@@ -743,14 +806,16 @@ static void execchild(struct ChanSess *chansess) {
 	/* clear environment */
 	/* if we're debugging using valgrind etc, we need to keep the LD_PRELOAD
 	 * etc. This is hazardous, so should only be used for debugging. */
-#ifndef DEBUG_KEEP_ENV
+#ifndef DEBUG_VALGRIND
 #ifdef HAVE_CLEARENV
 	clearenv();
 #else /* don't HAVE_CLEARENV */
-	environ = (char**)m_malloc(ENV_SIZE * sizeof(char*));
-	environ[0] = NULL;
+	/* Yay for posix. */
+	if (environ) {
+		environ[0] = NULL;
+	}
 #endif /* HAVE_CLEARENV */
-#endif /* DEBUG_KEEP_ENV */
+#endif /* DEBUG_VALGRIND */
 
 	/* We can only change uid/gid as root ... */
 	if (getuid() == 0) {
@@ -774,11 +839,18 @@ static void execchild(struct ChanSess *chansess) {
 		}
 	}
 
+	/* an empty shell should be interpreted as "/bin/sh" */
+	if (ses.authstate.pw->pw_shell[0] == '\0') {
+		usershell = "/bin/sh";
+	} else {
+		usershell = ses.authstate.pw->pw_shell;
+	}
+
 	/* set env vars */
 	addnewvar("USER", ses.authstate.pw->pw_name);
 	addnewvar("LOGNAME", ses.authstate.pw->pw_name);
 	addnewvar("HOME", ses.authstate.pw->pw_dir);
-	addnewvar("SHELL", ses.authstate.pw->pw_shell);
+	addnewvar("SHELL", usershell);
 	if (chansess->term != NULL) {
 		addnewvar("TERM", chansess->term);
 	}
@@ -797,22 +869,15 @@ static void execchild(struct ChanSess *chansess) {
 	agentset(chansess);
 #endif
 
-	/* an empty shell should be interpreted as "/bin/sh" */
-	if (ses.authstate.pw->pw_shell[0] == '\0') {
-		usershell = "/bin/sh";
-	} else {
-		usershell = ses.authstate.pw->pw_shell;
-	}
-
 	baseshell = basename(usershell);
 
 	if (chansess->cmd != NULL) {
 		argv[0] = baseshell;
 	} else {
 		/* a login shell should be "-bash" for "/bin/bash" etc */
-		argv[0] = (char*)m_malloc(strlen(baseshell) + 2); /* 2 for "-" */
-		strcpy(argv[0], "-");
-		strcat(argv[0], baseshell);
+		int len = strlen(baseshell) + 2; /* 2 for "-" */
+		argv[0] = (char*)m_malloc(len);
+		snprintf(argv[0], len, "-%s", baseshell);
 	}
 
 	if (chansess->cmd != NULL) {
@@ -824,7 +889,7 @@ static void execchild(struct ChanSess *chansess) {
 		argv[1] = NULL;
 	}
 
-	execv(ses.authstate.pw->pw_shell, argv);
+	execv(usershell, argv);
 
 	/* only reached on error */
 	dropbear_exit("child failed");
