@@ -56,11 +56,6 @@ int ecc_encrypt_key(const unsigned char *inkey, unsigned long keylen,
        return err;
     }
 
-    /* Encrypt the key */
-    for (x = 0; x < keylen; x++) {
-      skey[x] ^= inkey[x];
-    }
-
     /* output header */
     y = PACKET_SIZE;
  
@@ -78,9 +73,9 @@ int ecc_encrypt_key(const unsigned char *inkey, unsigned long keylen,
     STORE32L(keylen, out+y);
     y += 4;
 
-    /* Store the encrypted key */
+    /* Encrypt/Store the encrypted key */
     for (x = 0; x < keylen; x++, y++) {
-      out[y] = skey[x];
+      out[y] = skey[x] ^ inkey[x];
     }
 
     /* store header */
@@ -238,7 +233,7 @@ int ecc_sign_hash(const unsigned char *in,  unsigned long inlen,
       ecc_free(&pubkey);
       return CRYPT_MEM;
    }
-   if (mp_read_radix(&p, (unsigned char *)sets[key->idx].order, 64) != MP_OKAY)     { goto error; }
+   if (mp_read_radix(&p, (char *)sets[key->idx].order, 64) != MP_OKAY)     { goto error; }
    if (mp_read_unsigned_bin(&b, (unsigned char *)in, (int)inlen) != MP_OKAY)        { goto error; }
 
    /* find b = (m - x)/k */
@@ -324,7 +319,7 @@ int ecc_verify_hash(const unsigned char *sig, unsigned long siglen,
 
    _ARGCHK(sig != NULL);
    _ARGCHK(hash != NULL);
-   _ARGCHK(hash != NULL);
+   _ARGCHK(stat != NULL);
    _ARGCHK(key != NULL);
 
    /* default to invalid signature */
@@ -389,7 +384,7 @@ int ecc_verify_hash(const unsigned char *sig, unsigned long siglen,
    if (mp_read_unsigned_bin(&m, (unsigned char *)hash, (int)inlen) != MP_OKAY)     { goto error; }
    
    /* load prime */
-   if (mp_read_radix(&p, (unsigned char *)sets[key->idx].prime, 64) != MP_OKAY)    { goto error; }
+   if (mp_read_radix(&p, (char *)sets[key->idx].prime, 64) != MP_OKAY)    { goto error; }
    
    /* calculate barrett stuff */
    mp_set(&mu, 1); 
@@ -406,8 +401,8 @@ int ecc_verify_hash(const unsigned char *sig, unsigned long siglen,
    if (add_point(&pubkey.pubkey, &key->pubkey, &pubkey.pubkey, &p, &mu) != CRYPT_OK)    { goto error; }
 
    /* get mG */
-   if (mp_read_radix(&mG->x, (unsigned char *)sets[key->idx].Gx, 64) != MP_OKAY)   { goto error; }
-   if (mp_read_radix(&mG->y, (unsigned char *)sets[key->idx].Gy, 64) != MP_OKAY)   { goto error; }
+   if (mp_read_radix(&mG->x, (char *)sets[key->idx].Gx, 64) != MP_OKAY)   { goto error; }
+   if (mp_read_radix(&mG->y, (char *)sets[key->idx].Gy, 64) != MP_OKAY)   { goto error; }
    if (ecc_mulmod(&m, mG, mG, &p) != CRYPT_OK)                                     { goto error; }
 
    /* compare mG to bA + Y */
