@@ -26,25 +26,38 @@
 #define _OPTIONS_H_
 
 /******************************************************************
- * Define compile-time options below.
+ * Define compile-time options below - the "#ifndef DROPBEAR_XXX .... #endif"
+ * parts are to allow for commandline -DDROPBEAR_XXX options etc.
  ******************************************************************/
 
+#ifndef DROPBEAR_PORT
 #define DROPBEAR_PORT 22
+#endif
 
 /* Default hostkey paths - these can be specified on the command line */
-#define DSS_PRIV_FILENAME "/etc/dropbear_dss_host_key"
-#define RSA_PRIV_FILENAME "/etc/dropbear_rsa_host_key"
+#ifndef DSS_PRIV_FILENAME
+#define DSS_PRIV_FILENAME "/etc/dropbear/dropbear_dss_host_key"
+#endif
+#ifndef RSA_PRIV_FILENAME
+#define RSA_PRIV_FILENAME "/etc/dropbear/dropbear_rsa_host_key"
+#endif
 
-/* Disable X11 Forwarding */
-/*#define DISABLE_X11FWD*/
+/* Setting this disables the fast exptmod bignum code. It saves ~5kB, but is
+ * perhaps 20% slower for pubkey operations (it is probably worth experimenting
+ * if you want to use this) */
+/*#define NO_FAST_EXPTMOD*/
 
-/* Disable TCP Fowarding */
-/* NOTE: TCP forwarding is still work-in-progress, -L forwarding should work,
- * enable TCP forwarding at your own risk. */
-#define DISABLE_TCPFWD
+/* Enable X11 Forwarding */
+#define ENABLE_X11FWD
 
-/* Disable Authentication Agent Forwarding */
-/*#define DISABLE_AGENTFWD*/
+/* Enable TCP Fowarding */
+/* OpenSSH's "-L" style forwarding (client port forwarded via server) */
+#define ENABLE_LOCALTCPFWD
+/* OpenSSH's "-R" style forwarding (server port forwarded via client) */
+#define ENABLE_REMOTETCPFWD
+
+/* Enable Authentication Agent Forwarding */
+#define ENABLE_AGENTFWD
 
 /* Encryption - at least one required.
  * RFC Draft requires 3DES, and recommends Blowfish, AES128 & Twofish128 */
@@ -55,15 +68,22 @@
 
 /* Integrity - at least one required.
  * RFC Draft requires sha1-hmac, and recommends md5-hmac.
+ *
  * Note: there's no point disabling sha1 to save space, since it's used in the
  * for the random number generator and public-key cryptography anyway.
  * Disabling it here will just stop it from being used as the integrity portion
- * of the ssh protocol */
+ * of the ssh protocol.
+ *
+ * These are also used for key fingerprints in logs (when pubkey auth is used),
+ * MD5 fingerprints are printed by default, however SHA1 fingerprints will be
+ * generated otherwise. This isn't exactly optimal, although sha1 fingerprints
+ * are not too hard go create from pubkeys if required. */
 #define DROPBEAR_SHA1_HMAC
 #define DROPBEAR_MD5_HMAC
 
 /* Hostkey/public key algorithms - at least one required, these are used
  * for hostkey as well as for verifying signatures with pubkey auth.
+ * Removing either of these won't save very much space.
  * SSH2 RFC Draft requires dss, recommends rsa */
 #define DROPBEAR_RSA
 #define DROPBEAR_DSS
@@ -79,6 +99,15 @@
  * you are statically linking, you might want to undefine this */
 #define DO_HOST_LOOKUP
 
+/* Whether to print the message of the day (MOTD). This doesn't add much code
+ * size */
+#define DO_MOTD
+
+/* The MOTD file path */
+#ifndef MOTD_FILENAME
+#define MOTD_FILENAME "/etc/motd"
+#endif
+
 /* Authentication types to enable, at least one required.
    RFC Draft requires pubkey auth, and recommends password */
 #define DROPBEAR_PASSWORD_AUTH
@@ -94,40 +123,110 @@
 
 /*#undef DROPBEAR_PRNGD */ /* use prngd socket - you must manually set up prngd
 							  to produce output */
+#ifndef DROPBEAR_PRNGD_SOCKET
 #define DROPBEAR_PRNGD_SOCKET "/var/run/dropbear-rng"
+#endif
 
 /* Specify the number of clients we will allow to be connected but
  * not yet authenticated. After this limit, connections are rejected */
+#ifndef MAX_UNAUTH_CLIENTS
 #define MAX_UNAUTH_CLIENTS 30
+#endif
 
 /* Maximum number of failed authentication tries */
+#ifndef MAX_AUTH_TRIES
 #define MAX_AUTH_TRIES 10
+#endif
 
 /* The file to store the daemon's process ID, for shutdown scripts etc */
+#ifndef DROPBEAR_PIDFILE
 #define DROPBEAR_PIDFILE "/var/run/dropbear.pid"
+#endif
 
 /* The command to invoke for xauth when using X11 forwarding.
  * "-q" for quiet */
-#define XAUTH_COMMAND "/usr/bin/X11/xauth -q"
+#ifndef XAUTH_COMMAND
+#define XAUTH_COMMAND "/usr/X11R6/bin/xauth -q"
+#endif
+
+/* if you want to enable running an sftp server (such as the one included with
+ * OpenSSH), set the path below. If the path isn't defined, sftp will not
+ * be enabled */
+#ifndef SFTPSERVER_PATH
+#define SFTPSERVER_PATH "/usr/libexec/sftp-server"
+#endif
+
+/* This is used by the scp binary when used as a client binary */
+#define _PATH_SSH_PROGRAM "/usr/bin/ssh"
+
+/* Multi-purpose binary  configuration - if you want to make the combined
+ * binary, first define DROPBEAR_MULTI, and then define which of the three
+ * components you want. You should then compile Dropbear with 
+ * "make clean; make dropbearmulti". You'll need to install the binary
+ * manually, see MULTI for details */
+
+/* #define DROPBEAR_MULTI */
+
+/* The three multi binaries: dropbear, dropbearkey, dropbearconvert
+ * Comment out these if you don't want some of them */
+#define DBMULTI_DROPBEAR
+#define DBMULTI_KEY
+#define DBMULTI_CONVERT
+
 
 /*******************************************************************
  * You shouldn't edit below here unless you know you need to.
  *******************************************************************/
 
-#define DROPBEAR_VERSION "0.35"
+#ifndef DROPBEAR_VERSION
+#define DROPBEAR_VERSION "0.41"
+#endif
+
 #define LOCAL_IDENT "SSH-2.0-dropbear_" DROPBEAR_VERSION
 #define PROGNAME "dropbear"
 
-/* Time to wait before sending reply on fail */
-#define FAIL_SLEEP_TIME 2
+/* Spec recommends after one hour or 1 gigabyte of data. One hour
+ * is a bit too verbose, so we try 8 hours */
+#ifndef KEX_REKEY_TIMEOUT
+#define KEX_REKEY_TIMEOUT (3600 * 8)
+#endif
+#ifndef KEX_REKEY_DATA
+#define KEX_REKEY_DATA (1<<30) /* 2^30 == 1GB, this value must be < INT_MAX */
+#endif
+/* Close connections to clients which haven't authorised after AUTH_TIMEOUT */
+#ifndef AUTH_TIMEOUT
+#define AUTH_TIMEOUT 300 /* we choose 5 minutes */
+#endif
+
+/* Minimum key sizes for DSS and RSA */
+#ifndef MIN_DSS_KEYLEN
+#define MIN_DSS_KEYLEN 512
+#endif
+#ifndef MIN_RSA_KEYLEN
+#define MIN_RSA_KEYLEN 512
+#endif
+
+#define MAX_BANNER_SIZE 2000 /* this is 25*80 chars, any more is foolish */
+
+#define DEV_URANDOM "/dev/urandom"
+
+/* the number of NAME=VALUE pairs to malloc for environ, if we don't have
+ * the clearenv() function */
+#define ENV_SIZE 100
+
+#define MAX_CMD_LEN 1024 /* max length of a command */
+#define MAX_TERM_LEN 200 /* max length of TERM name */
+
+#define MAX_HOST_LEN 254 /* max hostname len for tcp fwding */
+#define MAX_IP_LEN 15 /* strlen("255.255.255.255") == 15 */
+
+#define DROPBEAR_MAX_PORTS 10 /* max number of ports which can be specified,
+								 ipv4 and ipv6 don't count twice */
+
+#define _PATH_TTY "/dev/tty"
 
 /* Timeouts in seconds */
 #define SELECT_TIMEOUT 20
-/* Spec recommends after one hour or 1 gigabyte of data */
-#define KEX_REKEY_TIMEOUT 3600
-#define KEX_REKEY_DATA (1<<30) /* 2^30 == 1GB, this value must be < INT_MAX */
-/* Close connections to clients which haven't authorised after AUTH_TIMEOUT */
-#define AUTH_TIMEOUT 600 /* 10 minutes is recommended */
 
 /* success/failure defines */
 #define DROPBEAR_SUCCESS 0
@@ -152,6 +251,8 @@
 #define SHA1_HASH_SIZE 20
 /* SHA512 is 64 bytes == 512 bits */
 #define SHA512_HASH_SIZE 64
+/* MD5 is 16 bytes = 128 bits */
+#define MD5_HASH_SIZE 16
 
 /* largest of MD5 and SHA1 */
 #define MAX_MAC_LEN SHA1_HASH_SIZE
@@ -181,26 +282,22 @@
 #define MAX_TRANS_WINDOW 500000000 /* 500MB is sufficient, stopping overflow */
 #define MAX_TRANS_WIN_INCR 500000000 /* overflow prevention */
 
-#define MAX_BANNER_SIZE 2000 /* this is 25*80 chars, any more is foolish */
+#define MAX_STRING_LEN 1400 /* ~= MAX_PROPOSED_ALGO * MAX_NAME_LEN, also
+							   is the max length for a password etc */
 
-#define DEV_URANDOM "/dev/urandom"
+#ifndef ENABLE_X11FWD
+#define DISABLE_X11FWD
+#endif
 
-/* the number of NAME=VALUE pairs to malloc for environ, if we don't have
- * the clearenv() function */
-#define ENV_SIZE 100
+#ifndef ENABLE_AGENTFWD
+#define DISABLE_AGENTFWD
+#endif
 
-#define MAX_CMD_LEN 1024 /* max length of a command */
-#define MAX_TERM_LEN 200 /* max length of TERM name */
-
-#define MAX_HOST_LEN 254 /* max hostname len for tcp fwding */
-
-#define DROPBEAR_MAX_PORTS 10 /* max number of ports which can be specified,
-								 ipv4 and ipv6 don't count twice */
-
-#define _PATH_TTY "/dev/tty"
-
-#ifdef DISABLE_TCPFWD
+#ifndef ENABLE_LOCALTCPFWD
 #define DISABLE_LOCALTCPFWD
+#endif
+
+#ifndef ENABLE_REMOTETCPFWD
 #define DISABLE_REMOTETCPFWD
 #endif
 
